﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace howto_stable_appointments
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }


        // Make some random preferences.
        private void Form1_Load(object sender, EventArgs e)
        {
            Random rnd = new Random();

            string txt = "";
            int pref = 0;
            for (int i = 1; i <= Person.DEFAULT_NUM_PEOPLE; i++)
            {
                txt += (char)((int)'A' + i - 1) + " ";

                for (int j = 1; j <= Person.NUM_PREFERENCES; j++)
                {
                    pref = (pref + rnd.Next(1, Person.NUM_CHOICES - j + 1)) % Person.NUM_CHOICES;
                    txt += pref + " ";
                }
                txt = txt + "\r\n";
            }

            txtPreferences.Text = txt;
            txtPreferences.Select(0, 0);
        }

        // Make assignments
        private void btnGo_Click(object sender, EventArgs e)
        {
            // Get the Persons' preferences.
            string txt = txtPreferences.Text;
            while (txt.EndsWith("\r\n"))
            {
                txt = txt.Substring(0, txt.Length - "\r\n".Length);
            }
            txt = txt.Replace("\r\n", "\n");
            string[] lines = txt.Split('\n');
            int num_people = lines.Length;

            Person[] people = new Person[num_people];
            for (int i = 0; i < num_people; i++)
                people[i] = new Person(lines[i]);

            // Clear assignments.
            Person[] assigned_to = new Person[Person.NUM_CHOICES];
            for (int i = 0; i < Person.NUM_CHOICES; i++)
            {
                assigned_to[i] = null;
            }

            // Make initial choice assignments.
            for (int pref = 0; pref < Person.NUM_PREFERENCES; pref++)
            {
                // Try to assign this choice for Persons.
                foreach (Person per in people)
                {
                    // See if this Person has an assignment yet.
                    if (per.Assignment < 0)
                    {
                        // This Person is unassigned.
                        // See if this choice is available.
                        int desired_choice = per.Preferences[pref];
                        if (assigned_to[desired_choice] == null)
                        {
                            // Assign this Person.
                            assigned_to[desired_choice] = per;
                            per.Assignment = desired_choice;
                        }
                    }
                }
            }

            // Assign anyone without an assignment.
            foreach (Person per in people)
            {
                // See if this Person has an assignment yet.
                if (per.Assignment < 0)
                {
                    // This Person is unassigned.
                    // Find an available choice.
                    for (int i = 0; i < Person.NUM_CHOICES; i++)
                    {
                        if (assigned_to[i] == null)
                        {
                            // Assign this Person.
                            assigned_to[i] = per;
                            per.Assignment = i;
                            break;
                        }
                    }
                }
            }

            // Try to improve the assignments.
            bool had_improvement;
            do
            {
                had_improvement = false;

                // Look for profitable swaps.
                foreach (Person per in people)
                {
                    foreach (Person per2 in people)
                    {
                        int per2_assignment = per2.Assignment;
                        int per_assignment = per.Assignment;

                        // See if per and per2 should swap.
                        int old_cost = per.Value + per2.Value;
                        int new_cost =
                            per.ValueOf(per2_assignment) +
                            per2.ValueOf(per_assignment);
                        if (new_cost < old_cost)
                        {
                            // Make the swap.
                            per.Assignment = per2_assignment;
                            per2.Assignment = per_assignment;
                            assigned_to[per_assignment] = per2;
                            assigned_to[per2_assignment] = per;
                            had_improvement = true;
                        }
                    }
                }
            } while (had_improvement);

            // Display the results.
            txtAssignments.Text = AssignmentSummary(assigned_to, people);
        }

        // Display the assignments.
        private string AssignmentSummary(Person[] assigned_to, Person[] people)
        {
            int total = 0;
            string txt = "Assignments" + "\r\n";
            for (int i = 0; i < assigned_to.Length; i++)
            {
                if (assigned_to[i] == null)
                {
                    txt += i + ": Nothing" + "\r\n";
                    total = total + Person.DONT_WANT;
                }
                else
                {
                    txt = txt + i + ": " + assigned_to[i].AssignmentString() + "\r\n";
                    total = total + assigned_to[i].Value;
                }
            }
            txt = txt + "----------" + "\r\n";

            txt = txt + "People" + "\r\n";
            total = 0;
            for (int i = 0; i < people.Length; i++)
            {
                txt = txt + people[i].Name + " " + people[i].Assignment +
                    " (" + people[i].Value + ")\r\n";
                total = total + people[i].Value;
            }
            txt = txt + "Total Value: " + total + "\r\n";

            return txt;
        }
    }
}
